@@ -33,7 +33,7 @@
 /**
  * SECTION:gbufferedinputstream
  * @short_description: Buffered Input Stream
- * @include: gio.h
+ * @include: gio/gio.h
  * @see_also: #GFilterInputStream, #GInputStream
  * 
  * Buffered input stream implements #GFilterInputStream and provides 
@@ -481,7 +481,7 @@ g_buffered_input_stream_fill_async (GBufferedInputStream *stream,
 					   user_data,
 					   G_IO_ERROR, G_IO_ERROR_INVALID_ARGUMENT,
 					   _("Too large count value passed to %s"),
-					   __FUNCTION__);
+					   G_STRFUNC);
       return;
     }
   
