@@ -30,7 +30,7 @@
 /**
  * SECTION:goutputstream
  * @short_description: Base class for implementing streaming output
- * @include: gio.h
+ * @include: gio/gio.h
  *
  *
  **/
@@ -157,7 +157,7 @@ g_output_stream_init (GOutputStream *stream)
  *
  * On success, the number of bytes written to the stream is returned.
  * It is not an error if this is not the same as the requested size, as it
- * can happen e.g. on a partial i/o error, or if the there is not enough
+ * can happen e.g. on a partial i/o error, or if there is not enough
  * storage in the stream. All writes either block until at least one byte
  * is written, so zero is never returned (unless @count is zero).
  * 
@@ -190,7 +190,7 @@ g_output_stream_write (GOutputStream  *stream,
   if (((gssize) count) < 0)
     {
       g_set_error (error, G_IO_ERROR, G_IO_ERROR_INVALID_ARGUMENT,
-		   _("Too large count value passed to %s"), __FUNCTION__);
+		   _("Too large count value passed to %s"), G_STRFUNC);
       return -1;
     }
 
@@ -645,7 +645,7 @@ g_output_stream_write_async (GOutputStream       *stream,
 					   user_data,
 					   G_IO_ERROR, G_IO_ERROR_INVALID_ARGUMENT,
 					   _("Too large count value passed to %s"),
-					   __FUNCTION__);
+					   G_STRFUNC);
       return;
     }
 
@@ -1080,7 +1080,10 @@ g_output_stream_set_pending (GOutputStream *stream,
   if (stream->priv->pending)
     {
       g_set_error (error, G_IO_ERROR, G_IO_ERROR_PENDING,
-		   _("Stream has outstanding operation"));
+		      /* Translators: This is an error you get if there is
+		       * already an operation running against this stream when
+		       * you try to start one */
+		      _("Stream has outstanding operation"));
       return FALSE;
     }
   
