@@ -22,6 +22,8 @@
 
 #include <config.h>
 
+#include <string.h>
+
 #include "gthemedicon.h"
 
 #include "gioalias.h"
@@ -29,7 +31,7 @@
 /**
  * SECTION:gthemedicon
  * @short_description: Icon theming support
- * @include: gio.h
+ * @include: gio/gio.h
  * @see_also: #GIcon, #GLoadableIcon
  *
  * #GThemedIcon is an implementation of #GIcon that supports icon themes.
@@ -141,6 +143,63 @@ g_themed_icon_new_from_names (char **iconnames,
   return G_ICON (themed);
 }
 
+/**
+ * g_themed_icon_new_with_default_fallbacks:
+ * @iconname: a string containing an icon name
+ *
+ * Creates a new themed icon for @iconname, and all the names
+ * that can be created by shortening @iconname at '-' characters.
+ * 
+ * In the following example, @icon1 and @icon2 are equivalent:
+ * |[
+ * const char *names[] = { 
+ *   "gnome-dev-cdrom-audio",
+ *   "gnome-dev-cdrom",
+ *   "gnome-dev",
+ *   "gnome"
+ * };
+ *
+ * icon1 = g_themed_icon_new_from_names (names, 4);
+ * icon2 = g_themed_icon_new_with_default_fallbacks ("gnome-dev-cdrom-audio");
+ * ]|
+ *
+ * Returns: a new #GThemedIcon.
+ */
+GIcon *
+g_themed_icon_new_with_default_fallbacks (const char *iconname)
+{
+  GThemedIcon *themed;
+  int i, dashes;
+  const char *p;
+  char *dashp;
+  char *last;
+
+  g_return_val_if_fail (iconname != NULL, NULL);
+  
+  themed = g_object_new (G_TYPE_THEMED_ICON, NULL);
+
+  dashes = 0;
+  p = iconname;
+  while (*p)
+    {
+      if (*p == '-')
+	dashes++;
+      p++;
+    }
+
+  themed->names = g_new (char *, dashes + 1 + 1);
+  i = 0;
+  themed->names[i++] = last = g_strdup (iconname);
+
+  while ((dashp = strrchr (last, '-')) != NULL)
+    themed->names[i++] = last = g_strndup (last, dashp - last);
+  
+  themed->names[i++] = NULL;
+
+  return G_ICON (themed);
+}
+
+
 /**
  * g_themed_icon_get_names:
  * @icon: a #GThemedIcon.
@@ -156,6 +215,28 @@ g_themed_icon_get_names (GThemedIcon *icon)
   return (const char * const *)icon->names;
 }
 
+/**
+ * g_themed_icon_append_name:
+ * @icon: a #GThemedIcon
+ * @iconname: name of icon to append to list of icons from within @icon.
+ *
+ * Append a name to the list of icons from within @icon.
+ */
+void
+g_themed_icon_append_name (GThemedIcon *icon, const char *iconname)
+{
+  guint num_names;
+  char **new_names;
+
+  g_return_if_fail (G_IS_THEMED_ICON (icon));
+  g_return_if_fail (iconname != NULL);
+
+  num_names = g_strv_length (icon->names);
+  icon->names = g_realloc (icon->names, sizeof (char*) * (num_names + 2));
+  icon->names[num_names] = g_strdup (iconname);
+  icon->names[num_names + 1] = NULL;
+}
+
 static guint
 g_themed_icon_hash (GIcon *icon)
 {
