@@ -57,10 +57,16 @@ unescape_character (const char *scanner)
  * g_uri_unescape_segment:
  * @escaped_string: a string.
  * @escaped_string_end: a string.
- * @illegal_characters: a string of illegal characters not to be allowed.
+ * @illegal_characters: an optional string of illegal characters not to be allowed.
  * 
  * Unescapes a segment of an escaped string.
- * 
+ *
+ * If any of the characters in @illegal_characters or the character zero appears
+ * as an escaped character in @escaped_string then that is an error and %NULL
+ * will be returned. This is useful it you want to avoid for instance having a
+ * slash being expanded in an escaped path element, which might confuse pathname
+ * handling.
+ *
  * Returns: an unescaped version of @escaped_string or %NULL on error.
  * The returned string should be freed when no longer needed.
  *
@@ -123,10 +129,16 @@ g_uri_unescape_segment (const char *escaped_string,
 /**
  * g_uri_unescape_string:
  * @escaped_string: an escaped string to be unescaped.
- * @illegal_characters: a string of illegal characters not to be allowed.
+ * @illegal_characters: an optional string of illegal characters not to be allowed.
  * 
  * Unescapes a whole escaped string.
  * 
+ * If any of the characters in @illegal_characters or the character zero appears
+ * as an escaped character in @escaped_string then that is an error and %NULL
+ * will be returned. This is useful it you want to avoid for instance having a
+ * slash being expanded in an escaped path element, which might confuse pathname
+ * handling.
+ *
  * Returns: an unescaped version of @escaped_string. The returned string 
  * should be freed when no longer needed.
  *
@@ -140,10 +152,10 @@ g_uri_unescape_string (const char *escaped_string,
 }
 
 /**
- * g_uri_get_scheme:
+ * g_uri_parse_scheme:
  * @uri: a valid URI.
  * 
- * Gets the scheme portion of a URI. RFC 3986 decodes the scheme as:
+ * Gets the scheme portion of a URI string. RFC 3986 decodes the scheme as:
  * <programlisting>
  * URI = scheme ":" hier-part [ "?" query ] [ "#" fragment ] 
  * </programlisting>
@@ -155,7 +167,7 @@ g_uri_unescape_string (const char *escaped_string,
  * Since: 2.16
  **/
 char *
-g_uri_get_scheme (const char  *uri)
+g_uri_parse_scheme (const char  *uri)
 {
   const char *p;
   char c;
@@ -192,6 +204,15 @@ g_uri_get_scheme (const char  *uri)
   return g_strndup (uri, p - uri - 1);
 }
 
+/* Temporary add a symbol to avoid crashing to many apps.
+   Remove after 2.15.6 release */
+char * g_uri_get_scheme (const char  *uri);
+char *g_uri_get_scheme (const char  *uri)
+{
+  return g_uri_parse_scheme (uri);
+}
+
+
 /**
  * g_uri_escape_string:
  * @unescaped: the unescaped input string.
